package sun.java2d.pipe;

import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.awt.BasicStroke;
import java.awt.Shape;
import sun.java2d.SunGraphics2D;

public abstract class SpanShapeRenderer implements ShapeDrawPipe
{
    static final RenderingEngine RenderEngine;
    public static final int NON_RECTILINEAR_TRANSFORM_MASK = 48;
    
    public SpanShapeRenderer() {
    }
    
    @Override
    public void draw(SunGraphics2D sunGraphics2D, Shape shape) {
        if ((sunGraphics2D.stroke instanceof BasicStroke)) {
            ShapeSpanIterator strokeSpans = LoopPipe.getStrokeSpans(sunGraphics2D, shape);
            try {
                this.renderSpans(sunGraphics2D, sunGraphics2D.getCompClip(), shape, strokeSpans);
            }
            finally {
                strokeSpans.dispose();
            }
        }
        else {
            this.fill(sunGraphics2D, sunGraphics2D.stroke.createStrokedShape(shape));
        }
    }
    
    @Override
    public void fill(SunGraphics2D sunGraphics2D, Shape shape) {
        if (shape instanceof Rectangle2D && (sunGraphics2D.transform.getType() & 0x30) == 0x0) {
            this.renderRect(sunGraphics2D, (Rectangle2D)shape);
            return;
        }
        Region compClip = sunGraphics2D.getCompClip();
        ShapeSpanIterator fillSSI = LoopPipe.getFillSSI(sunGraphics2D);
        try {
            fillSSI.setOutputArea(compClip);
            fillSSI.appendPath(shape.getPathIterator(sunGraphics2D.transform));
            this.renderSpans(sunGraphics2D, compClip, shape, fillSSI);
        }
        finally {
            fillSSI.dispose();
        }
        return;
    }
    
    public abstract Object startSequence(SunGraphics2D p0, Shape p1, Rectangle p2, int[] p3);
    
    public abstract void renderBox(Object p0, int p1, int p2, int p3, int p4);
    
    public abstract void endSequence(Object p0);
    
    public void renderRect(SunGraphics2D sunGraphics2D, Rectangle2D rectangle2D) {
        double[] array = { rectangle2D.getX(), rectangle2D.getY(), rectangle2D.getWidth(), rectangle2D.getHeight() };
        int n = 2;
        array[n] += array[0];
        int n2 = 3;
        array[n2] += array[1];
        if (array[2] <= array[0] || array[3] <= array[1]) {
            return;
        }
        sunGraphics2D.transform.transform(array, 0, array, 0, 2);
        if (array[2] < array[0]) {
            double n3 = array[2];
            array[2] = array[0];
            array[0] = n3;
        }
        if (array[3] < array[1]) {
            double n4 = array[3];
            array[3] = array[1];
            array[1] = n4;
        }
        int[] array2 = { (int)array[0], (int)array[1], (int)array[2], (int)array[3] };
        Rectangle o = new Rectangle(array2[0], array2[1], array2[2] - array2[0], array2[3] - array2[1]);
        Region compClip = sunGraphics2D.getCompClip();
        compClip.clipBoxToBounds(array2);
        if (array2[0] >= array2[2] || array2[1] >= array2[3]) {
            return;
        }
        Object startSequence = this.startSequence(sunGraphics2D, rectangle2D, o, array2);
        if (compClip.isRectangular()) {
            this.renderBox(startSequence, array2[0], array2[1], array2[2] - array2[0], array2[3] - array2[1]);
        }
        else {
            while (compClip.getSpanIterator(array2).nextSpan(array2)) {
                this.renderBox(startSequence, array2[0], array2[1], array2[2] - array2[0], array2[3] - array2[1]);
            }
        }
        this.endSequence(startSequence);
        return;
    }
    
    public void renderSpans(SunGraphics2D sunGraphics2D, Region region, Shape shape, ShapeSpanIterator shapeSpanIterator) {
        Object startSequence = null;
        int[] array = new int[4];
        try {
            shapeSpanIterator.getPathBox(array);
            Rectangle rectangle = new Rectangle(array[0], array[1], array[2] - array[0], array[3] - array[1]);
            region.clipBoxToBounds(array);
            if (array[0] >= array[2] || array[1] >= array[3]) {
                return;
            }
            shapeSpanIterator.intersectClipBox(array[0], array[1], array[2], array[3]);
            startSequence = this.startSequence(sunGraphics2D, shape, rectangle, array);
            this.spanClipLoop(startSequence, shapeSpanIterator, region, array);
        }
        finally {
            if (startSequence != null) {
                this.endSequence(startSequence);
            }
        }
    }
    
    public void spanClipLoop(Object o, SpanIterator filter, Region region, int[] array) {
        if (!region.isRectangular()) {
            filter = region.filter(filter);
        }
        while (filter.nextSpan(array)) {
            int n = array[0];
            int n2 = array[1];
            this.renderBox(o, n, n2, array[2] - n, array[3] - n2);
        }
    }
    
    static {
        RenderEngine = RenderingEngine.getInstance();
    }
    
    public static class Composite extends SpanShapeRenderer
    {
        CompositePipe comppipe;
        
        public Composite(CompositePipe comppipe) {
            this.comppipe = comppipe;
        }
        
        @Override
        public Object startSequence(SunGraphics2D sunGraphics2D, Shape shape, Rectangle rectangle, int[] array) {
            return this.comppipe.startSequence(sunGraphics2D, shape, rectangle, array);
        }
        
        @Override
        public void renderBox(Object o, int n, int n2, int n3, int n4) {
            this.comppipe.renderPathTile(o, null, 0, n3, n, n2, n3, n4);
        }
        
        @Override
        public void endSequence(Object o) {
            this.comppipe.endSequence(o);
        }
    }
    
    public static class Simple extends SpanShapeRenderer implements LoopBasedPipe
    {
        @Override
        public Object startSequence(SunGraphics2D sunGraphics2D, Shape shape, Rectangle rectangle, int[] array) {
            return sunGraphics2D;
        }
        
        @Override
        public void renderBox(Object o, int n, int n2, int n3, int n4) {
            SunGraphics2D sunGraphics2D = (SunGraphics2D)o;
            sunGraphics2D.loops.fillRectLoop.FillRect(sunGraphics2D, sunGraphics2D.getSurfaceData(), n, n2, n3, n4);
        }
        
        @Override
        public void endSequence(Object o) {
        }
    }
}
